# distutils: language=c++

import numpy as np
cimport numpy as np
cimport cython
import cython


cdef extern from "kmeansinplace.hpp":
    void kmeans_c(double *points, int m, int n, int *result, int k)
    void kmeans_greedy_c(double *points, int m, int n, int *result, int k)
    void kmeans_greedy_c_norm1(double *points, int m, int n, int *result, int k)


@cython.boundscheck(False)
@cython.wraparound(False)
def kmeansinplace_init(np.ndarray[double, ndim=2, mode="c"] input not None,
                   np.ndarray[int, ndim=1, mode="c"] output not None, int k):
    cdef int m, n
    m, n = input.shape[0], input.shape[1]
    assert output.shape[0] == m, f"Output should have shape {m,}, but got {output.shape[0]}"
    kmeans_c(&input[0,0], m, n, &output[0], k)

    return None


@cython.boundscheck(False)
@cython.wraparound(False)
def kmeansinplace_init_greedy(np.ndarray[double, ndim=2, mode="c"] input not None,
                   np.ndarray[int, ndim=1, mode="c"] output not None, int k,
                              int norm):
    cdef int m, n
    m, n = input.shape[0], input.shape[1]
    assert output.shape[0] == m, f"Output should have shape {m,}, but got {output.shape[0]}"

    if norm == 2:
        kmeans_greedy_c(&input[0,0], m, n, &output[0], k)
    elif norm == 1:
        kmeans_greedy_c_norm1(&input[0,0], m, n, &output[0], k)
    else:
        raise ValueError(f'Only norm 1 and 2 are supported. Got: norm={norm}.')

    return None